/**
 * OW2 FraSCAti: SCA Implementation Fractal
 * Copyright (C) 2008-2010 INRIA, University of Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Damien Fournier
 *
 * Contributor(s): Philippe Merle
 *
 */

package org.ow2.frascati.implementation.fractal;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.stp.sca.domainmodel.frascati.FrascatiPackage;
import org.eclipse.stp.sca.domainmodel.frascati.FractalImplementation;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.FactoryFactory;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.Factory;

import org.objectweb.fractal.julia.Julia;                                                                                                                               

import org.ow2.frascati.assembly.factory.api.ProcessingContext;
import org.ow2.frascati.assembly.factory.api.ProcessorException;
import org.ow2.frascati.assembly.factory.processor.AbstractImplementationProcessor;

/**
 * OW2 FraSCAti SCA Implementation Fractal processor class.
 *
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version 1.3
 */
public class FrascatiImplementationFractalProcessor
     extends AbstractImplementationProcessor<FractalImplementation>
{
  //---------------------------------------------------------------------------
  // Internal state.
  // --------------------------------------------------------------------------

  static {
    System.setProperty("fractal.provider", Julia.class.getCanonicalName());
  }

  //---------------------------------------------------------------------------
  // Internal methods.
  // --------------------------------------------------------------------------

  /**
   * @see org.ow2.frascati.assembly.factory.processor.AbstractProcessor#toStringBuffer(EObjectType, StringBuffer)
   */
  @Override
  protected final void toStringBuffer(FractalImplementation fractalImplementation, StringBuffer sb)
  {
    sb.append("frascati:implementation.fractal");
    append(sb, "definition", fractalImplementation.getDefinition());
    super.toStringBuffer(fractalImplementation, sb);
  }

  /**
   * @see org.ow2.frascati.assembly.factory.api.Processor#check(ElementType, ProcessingContext)
   */
  @Override
  protected final void doCheck(FractalImplementation fractalImplementation, ProcessingContext processingContext)
      throws ProcessorException
  {
    String fractalImplementationDefinition = fractalImplementation.getDefinition();
    if(fractalImplementationDefinition == null || fractalImplementationDefinition.equals("")) {
      processingContext.error(toString(fractalImplementation) + " the attribute 'definition' must be set");
    } else {
      int index = fractalImplementationDefinition.indexOf('(');
      String fractalFile = (index == -1)
                         ? fractalImplementationDefinition
                         : fractalImplementationDefinition.substring(0, index);
      if(processingContext.getResource(fractalFile.replace(".", "/") + ".fractal") == null) {
        processingContext.error(toString(fractalImplementation) + " definition '" + fractalFile + "' not found");
      }
    }

    // check attributes 'policySets' and 'requires'.
    checkImplementation(fractalImplementation, processingContext);
  }

  /**
   * @see org.ow2.frascati.assembly.factory.api.Processor#generate(ElementType, ProcessingContext)
   */
  @Override
  protected final void doGenerate(FractalImplementation fractalImplementation, ProcessingContext processingContext)
      throws ProcessorException
  {
    logFine(fractalImplementation, "nothing to generate");
  }

  /**
   * @see org.ow2.frascati.assembly.factory.api.Processor#instantiate(ElementType, ProcessingContext)
   */
  @Override
  protected final void doInstantiate(FractalImplementation fractalImplementation, ProcessingContext processingContext)
      throws ProcessorException
  {
    String definition = fractalImplementation.getDefinition();

    Component component;
    try {
      // Try loading from definition generated with Juliac
      Class<?> javaClass = processingContext.loadClass(definition);
      Factory object = (Factory) javaClass.newInstance();

      // Create the component instance.
      component = object.newFcInstance();

    } catch (ClassNotFoundException e) {

      // Generated class not found try with fractal definition and Fractal ADL
      org.objectweb.fractal.adl.Factory f;

      try {
        // init Fractal ADL factory
        Map<Object, Object> context = new HashMap<Object, Object>();
        context.put("classloader", processingContext.getClassLoader());
        f = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND);

        // try instanciation for the given Fractal ADL definition
        component = (Component) f.newComponent(definition, context);

      } catch (ADLException ae) {
        // Error with Fractal ADL
        severe(new ProcessorException(fractalImplementation, "Error in the Fractal implementation : "
            + definition, ae));
        return;
      }
    } catch (InstantiationException ie) {
      severe(new ProcessorException(fractalImplementation,
          "Error when building instance for class " + definition, ie));
      return;
    } catch (IllegalAccessException iae) {
      severe(new ProcessorException(fractalImplementation, "Could not access class "
          + definition, iae));
      return;
    } catch (org.objectweb.fractal.api.factory.InstantiationException ie) {
      severe(new ProcessorException(fractalImplementation,
          "Error when building component instance : " + definition, ie));
      return;
    }
 
    // Store the created component into the processing context.
    processingContext.putData(fractalImplementation, Component.class, component);
  }

  //---------------------------------------------------------------------------
  // Public methods.
  // --------------------------------------------------------------------------

  /**
   * @see org.ow2.frascati.assembly.factory.api.Processor#getProcessorID()
   */
  public final String getProcessorID()
  {
    return getID(FrascatiPackage.Literals.FRACTAL_IMPLEMENTATION);
  }

}
